unit ContextM;

interface

uses
   Windows, ActiveX, ComObj, ShlObj, Dialogs;

type
   TContextMenu = class(TComObject, IShellExtInit, IContextMenu)
   private
      FFileName: array[0..MAX_PATH] of Char;
   protected
    { IShellExtInit }
      function IShellExtInit.Initialize = SEIInitialize; // Avoid compiler warning
      function SEIInitialize(pidlFolder: PItemIDList; lpdobj: IDataObject;
         hKeyProgID: HKEY): HResult; stdcall;
    { IContextMenu }
      function QueryContextMenu(Menu: HMENU; indexMenu, idCmdFirst, idCmdLast,
         uFlags: UINT): HResult; stdcall;
      function InvokeCommand(var lpici: TCMInvokeCommandInfo): HResult; stdcall;
      function GetCommandString(idCmd, uType: UINT; pwReserved: PUINT;
         pszName: LPSTR; cchMax: UINT): HResult; stdcall;
   end;

const
   Class_ContextMenu: TGUID = '{a2451ab0-8667-49ea-8709-7710f434031e}';

implementation

uses ComServ, SysUtils, ShellApi, Registry, Math, Forms,
   Progress, CPSelect, CodeConvTools;
var
   MyCodePages: array[tCodePage] of integer;


function TContextMenu.SEIInitialize(pidlFolder: PItemIDList; lpdobj: IDataObject;
   hKeyProgID: HKEY): HResult;
var
   StgMedium: TStgMedium;
   FormatEtc: TFormatEtc;
begin
  // Fail the call if lpdobj is Nil.
   if (lpdobj = nil) then
   begin
      Result := E_INVALIDARG;
      Exit;
   end;

   with FormatEtc do
   begin
      cfFormat := CF_HDROP;
      ptd := nil;
      dwAspect := DVASPECT_CONTENT;
      lindex := -1;
      tymed := TYMED_HGLOBAL;
   end;

  // Render the data referenced by the IDataObject pointer to an HGLOBAL
  // storage medium in CF_HDROP format.
   Result := lpdobj.GetData(FormatEtc, StgMedium);
   if Failed(Result) then
      Exit;
  // If only one file is selected, retrieve the file name and store it in
  // FFileName. Otherwise fail the call.
   if (DragQueryFile(StgMedium.hGlobal, $FFFFFFFF, nil, 0) = 1) then
   begin
      DragQueryFile(StgMedium.hGlobal, 0, FFileName, SizeOf(FFileName));
      Result := NOERROR;
   end
   else
   begin
      FFileName[0] := #0;
      Result := E_FAIL;
   end;
   ReleaseStgMedium(StgMedium);
end;

function TContextMenu.QueryContextMenu(Menu: HMENU; indexMenu, idCmdFirst,
   idCmdLast, uFlags: UINT): HResult;
var
   Exts: string;
//   AnyCP: integer;
//   CascadeMenu: integer;
   CodePages: array[tCodePage] of byte;
   CP: tCodePage;
   NewMenuItem: array[0..50] of char;
   CPName: array[0..30] of char;
   MyCommand: uint;
begin
   Result := 0; // or use MakeResult(SEVERITY_SUCCESS, FACILITY_NULL, 0);
   for cp := FirstCodePage to cpAny do
      MyCodePages[cp] := -1;
   if ((uFlags and $0000000F) = CMF_NORMAL) or
      ((uFlags and CMF_EXPLORE) <> 0) then
   begin
      with TRegistry.Create do
      try
         RootKey := HKEY_CURRENT_USER;
         OpenKey('SOFTWARE\CICOM\CodeConv\2.0\ShellExt', False);
         Exts := ReadString('Extensions');
//         CascadeMenu := ReadInteger('CascadeMenu');
         ReadBinaryData('CodePages', Codepages, SizeOf(CodePages));
      finally
         Free;
      end;
      if pos(LowerCase(ExtractFileExt(string(FFileName))), Exts) > 0 then
      begin
    // Add one menu item to context menu
         MyCommand := idCmdFirst;
         for cp := cpAny downto FirstCodePage do
            if CodePages[cp] <> 0 then
            begin
               if MyCommand > idCmdLast then
               begin ShowMessage('CodeConv Shell Extensions Error:' + #13 +
                     'Not enought free command IDs to display all wanted manuitems!');
                  exit;
               end;
               if cp = cpAny then
                  NewMenuItem := 'Change code page...'
               else
               begin
                  StrPCopy(CPName, CodePageNames[cp]);
                  NewMenuItem := 'Change code page to ';
                  StrCat(NewMenuItem, CPName);
               end;
               InsertMenu(Menu, indexMenu, MF_STRING or MF_BYPOSITION, MyCommand, NewMenuItem);
               MyCodePages[cp] := MyCommand - idCmdFirst;
               inc(MyCommand);
               inc(Result);
            end;
         if Result > 0 then
         begin
            InsertMenu(Menu, indexMenu, MF_SEPARATOR or MF_BYPOSITION, 0, nil);
            inc(Result);
         end;
    // Return number of menu items added
      end;
   end;
end;

function TContextMenu.InvokeCommand(var lpici: TCMInvokeCommandInfo): HResult;
{var
  H: THandle;}
var
   i: integer;
   cp, cp0: tCodePage;
   s: array[0..512] of char;
   Ask: integer;
begin

   if (HiWord(Integer(lpici.lpVerb)) <> 0) then
   begin
      Result := E_FAIL;
      Exit;
   end;
   i := -1;
   with TRegistry.Create do
   try
      RootKey := HKEY_CURRENT_USER;
      OpenKey('SOFTWARE\CICOM\CodeConv\2.0\ShellExt', False);
      Ask := ReadInteger('Ask');
   finally
      Free;
   end;
   for cp := FirstCodePage to cpAny do
      i := Max(MyCodePages[cp], i);
   if LoWord(lpici.lpVerb) > i then
   begin
      Result := E_INVALIDARG;
      Exit;
   end
   else
   begin
      for cp := FirstCodePage to cpAny do
         if MyCodePages[cp] = LoWord(lpici.lpVerb) then break;
      if cp = cpAny then
      begin
         try
            CodeConvShellExt := TCodeConvShellExt.Create(nil);
            CodeConvShellExt.FileLabel.Caption := FFileName;
            CodeConvShellExt.cpFrom := DetectCodePage(FFileName);
            CodeConvShellExt.CPLabel.Caption := CodePageNames[CodeConvShellExt.cpFrom];
            CodeConvShellExt.ParentWindow := 0;
            CodeConvShellExt.Parent := nil;
            Application.Handle := 0;
            CodeConvShellExt.ShowModal;
         finally
            CodeConvShellExt.Free;
         end;
      end
      else
      begin
         s := '';
         strpcopy(s, 'Do you want to convert file ' + FFileName + #13 + 'to ' + CodePageNames[cp] +
            ' code page?');
         if (Ask = 0) or (Windows.MessageBox(0, s, 'CodeConv Shell Extensions', MB_YESNO +
            MB_ICONQUESTION + MB_DEFBUTTON1) = IDYES) then
         begin
            try
               ProgressForm := TProgressForm.Create(nil);
               Application.Handle := 0;
               ProgressForm.ParentWindow := 0;
               ProgressForm.Parent := nil;
               ProgressForm.Gauge.Progress := 0;
               ProgressForm.Show;
               ProgresProc := ProgressForm.Make;
               cp0 := DetectCodePage(FFileName);
               if ConvertFile(FFileName, FFileName, cp0, cp, CaseNoChange, CRLFMakeAsCP) <= 0 then
                  ShowMessage('CodeConv Shell Extensions Error:' + #13 +
                     'Some error occured while converting.');
               ProgressForm.Close;
            finally
               ProgressForm.Free;
            end;
         end;
      end;
   end;

   Result := NOERROR;
end;

function TContextMenu.GetCommandString(idCmd, uType: UINT; pwReserved: PUINT;
   pszName: LPSTR; cchMax: UINT): HRESULT;
begin
   if (idCmd = 0) then
   begin
      if (uType = GCS_HELPTEXT) then
      // return help string for menu item
         StrCopy(pszName, 'Convert code page of text file');
      Result := NOERROR;
   end
   else
      if (idCmd = 1) then
      begin
         if (uType = GCS_HELPTEXT) then
      // return help string for menu item
            StrCopy(pszName, 'Convert code page of text file to Windows');
         Result := NOERROR;
      end
      else
         Result := E_INVALIDARG;
end;

type
   TContextMenuFactory = class(TComObjectFactory)
   public
      procedure UpdateRegistry(Register: Boolean); override;
   end;

procedure TContextMenuFactory.UpdateRegistry(Register: Boolean);
var
   ClassID: string;
begin
   if Register then
   begin
      inherited UpdateRegistry(Register);

      ClassID := GUIDToString(Class_ContextMenu);
      CreateRegKey('*\shellex\ContextMenuHandlers\CodeConv', '', ClassID);

      if (Win32Platform = VER_PLATFORM_WIN32_NT) then
         with TRegistry.Create do
         try
            RootKey := HKEY_LOCAL_MACHINE;
            OpenKey('SOFTWARE\Microsoft\Windows\CurrentVersion\Shell Extensions', True);
            OpenKey('Approved', True);
            WriteString(ClassID, 'CodeConv Shell Extensions');
         finally
            Free;
         end;
   end
   else
   begin
      with TRegistry.Create do
      try
         RootKey := HKEY_CLASSES_ROOT;
         DeleteKey('\*\shellex\ContextMenuHandlers\CodeConv');
         OpenKey('\*\shellex\ContextMenuHandlers', False);
         if not (HasSubKeys) then DeleteKey('\*\shellex\ContextMenuHandlers');
         OpenKey('\*\shellex', False);
         if not (HasSubKeys) then DeleteKey('\*\shellex');
         OpenKey('\*', False);
         if not (HasSubKeys) then DeleteKey('\*');
      finally
         Free;
      end;
      inherited UpdateRegistry(Register);
   end;
end;

initialization
   TContextMenuFactory.Create(ComServer, TContextMenu, Class_ContextMenu,
      '', 'CodeConv Shell Extensions', ciMultiInstance,
      tmApartment);
end.

